#!/usr/bin/python
import web, sys, gntp.notifier, urllib2
import ConfigParser,os, ast, string, json
from collections import defaultdict

urls = ('/.*', 'hooks')
listenport = 11001
config = ConfigParser.ConfigParser()
templates = {}
defaultIcon = os.path.join(os.path.dirname(sys.argv[0]),'icons','webhook.png')
growl = ""
configurationfile = os.path.join(os.path.dirname(sys.argv[0]),'webhooks.ini')
defaultConfigFile = configurationfile

# default configuration file locations
try:
	if sys.platform == 'win32':
		configurationfile = os.path.join(os.environ['APPDATA'],'webhook_gntp_gateway','webhooks.ini')
	elif sys.platform == 'linux2':
		configurationfile = os.path.join(os.environ['HOME'],'webhook_gntp_gateway','.webhooks.ini')
	with open(configurationfile): pass
except IOError,KeyError:
	configurationfile = defaultConfigFile

# Handle arguments
for x in sys.argv:
	if x in ['help','-h','-?','?']:
		print '\n' + sys.argv[0] + '\nWebhook to GNTP gateway\n' \
		'Creates a minimal webserver to listen for webhook requests\n' \
		'and forward them as GNTP (growl) messages\n' \
		'(c) Hugo Ahlenius 2013 http://nordpil.com\n' \
		'Apache license\n\nArguments: [full path to .ini file for configuration]' \
		'(defaults to webhooks.ini in %appdata%\webhook_gntp_gateway or same folder)\n'
		sys.exit()
	if x[-3:] == 'ini':
		configurationfile = x

# check configuration file
try:
	with open(configurationfile): pass
except:
	print '\nUnable to open configuration file, please check that file\n' \
	'' + configurationfile + '\n' \
	'exists and is readable by ' + sys.argv[0]
	sys.exit()

class MyApplication(web.application):
	def run(self, port=listenport, *middleware):
		func = self.wsgifunc(*middleware)
		return web.httpserver.runsimple(func, ('0.0.0.0', listenport))

class hooks:
	def GET(self):
		print web.url()
		return 'OK'

	def POST(self):
		global templates
		global defaultIcon
		thisApp = web.url().strip('/')
		if thisApp in templates:
			print web.data()
			if templates[thisApp]['type'] == 'postval':
				data = web.data()
				data = dict([y.split('=') for y in data.split('&')])
				for x in data:
					data[x] = urllib2.unquote(data[x])
				title = string.Formatter().vformat(templates[thisApp]['title'],(),data)
				description = string.Formatter().vformat('\n'.join(templates[thisApp]['description'].split('\\n')),(),data)
				try:
					noteImagePath = os.path.join(os.path.dirname(sys.argv[0]),string.Formatter().vformat(templates[thisApp]['icon'],(),data))
					noteImage = open(noteImagePath,'rb').read()
				except:
					noteImage = open(defaultIcon,'rb').read()
			elif templates[thisApp]['type'] == 'json':
				# TODO: For json/newrelic, we are basically discarding the opening variable/type...
				data = urllib2.unquote(web.data())
				data = data.split('=')[1]
				data = json.loads(data)
				data = defaultdict(str,data)
				title = string.Formatter().vformat(templates[thisApp]['title'],(),data)
				description = string.Formatter().vformat('\n'.join(templates[thisApp]['description'].split('\\n')),(),data)
				try:
					noteImagePath = os.path.join(os.path.dirname(sys.argv[0]),string.Formatter().vformat(templates[thisApp]['icon'],(),data))
					noteImage = open(noteImagePath,'rb').read()
				except:
					noteImage = open(defaultIcon,'rb').read()

			print data
			growl.notify(
				noteType = 'Webhook notification',
				title = title,
				description = description,
				icon = noteImage,
				sticky = False,
				priority = 1
			)
		else:
			# If no template is defined for this appKey
			print web.data()
			print defaultIcon
			noteImage = open(defaultIcon,'rb').read()
			growl.notify(
				noteType = 'Webhook notification',
				title = 'No template defined for app ' + thisApp,
				description = '%s\n\n%s' % (web.url(), web.data()),
				icon = open(defaultIcon,'rb').read(),
				sticky = False,
				priority = 1
			)
		return 'OK'

class readConfiguration():
	global listenport
	global templates
	global growl
	global configurationfile
	global defaultIcon

	try:
		config.read(configurationfile)
		listenport = int(config.get('GENERAL','port'))
		for section in config.sections():
			if 'appkey' in config.options(section):
				thisApp = config.get(section,'appkey')
				templates[thisApp] = {}
				for item in ['type','title','description','icon']:
					templates[thisApp][item] = config.get(section,item)
		applicationName = config.get('GENERAL','applicationname')
		growlhost = config.get('GENERAL','growlhost')
		try:
			growlpass = config.get('GENERAL','growlpass')
		except:
			growlpass = ''
	except:
		print '\nUnable to parse configuration file, please check that file\n' \
		'' + configurationfile + '\n' \
		'conforms to the specification'
		sys.exit()

	try:
		growl = gntp.notifier.GrowlNotifier(
			applicationName = applicationName,
			notifications = ["Webhook notification"],
			defaultNotifications = ["Webhook notification"],
			hostname = growlhost,
			password = growlpass
		)
		growl.register()
		growl.notify(
			noteType = 'Webhook notification',
			title = applicationName,
			description = 'Now listening for webhooks on\nport ' + str(listenport),
			icon = open(defaultIcon,'rb').read(),
			sticky = False,
			priority = 1
		)
	except:
		print sys.exc_info()
		print '\nUnable to register and communicate with the specified\n' \
		'growl instance, please verify the settings in ' + configurationfile
		sys.exit()
		

if __name__ == '__main__':
	app = MyApplication(urls, globals())
	app.run(port=listenport)
